/**
 * Priority queue.
 *
 * Priority queue can insert elements within the complexity of log(N) or remove
 * the largest element in the queue.
 */
export interface PriorityQueue<T> {
    /**
     * Init priority queue with initial elements.
     * @param elements
     */
    init(elements: ReadonlyArray<T>): void;
    /**
     * Drop a element into the priority queue.
     */
    enqueue(val: T): void;
    /**
     * Popup the top element.
     */
    dequeue(): T | undefined;
    /**
     * Get the top element.
     */
    top(): T | undefined;
    /**
     * Return the number of elements of the priority queue.
     */
    size(): number;
    /**
     * Check if the priority queue is empty.
     */
    isEmpty(): boolean;
    /**
     * Return all of the elements of the priority queue.
     */
    collect(): T[];
}
/**
 * Create a priority queue, implementation based on a max-heap.
 *
 * @param cmp   Comparison function, if the result > 0, the element on the left
 *              side of the operator has higher precedence.
 * @returns
 */
export declare function createPriorityQueue<T>(cmp: (x: T, y: T) => -1 | 0 | 1 | number): PriorityQueue<T>;
